/*
Use me code, state machine for keeping track of dallas temp sensors
Set up to not be blocking, but just maintain a temperature reading(s) other software can read at will
Doug Coulter 2015 GPLV2 license
*/

#include <OneWire.h>
//#define REQUIRESALARMS false
#include <DallasTemperature.h>
// Data wire is plugged into pin 2 on the Arduino
#define ONE_WIRE_BUS 2
// Setup a oneWire instance to communicate with any OneWire devices (not just Maxim/Dallas temperature ICs)
OneWire oneWire(ONE_WIRE_BUS);
// Pass our oneWire reference to Dallas Temperature. 
DallasTemperature sensors(&oneWire);
#define D_MILLIS 1000 // wait time after start conversion
unsigned long Dtime;  // last start time, millis
byte Ddevices;    // number of Dallas devices detected
byte i; // general purpose iterator
byte dstate;      // state variable
float DtempsF[8]; // library supports up to 8 devices on the wire..
byte Dflag;
/////////////////////////
void setup_DS18B20()
{
 // Start up the library
  sensors.begin();  // Get going
  sensors.setResolution(12);  // max res
  Ddevices = sensors.getDeviceCount(); // how many we have
  sensors.setWaitForConversion(false); // we'll do our own timing  
  Dflag = dstate = 0;
}
void Dstate()
{
  switch (dstate)
  {
   case 0:
   sensors.requestTemperatures(); // Send the command to get temperatures
   Dtime = millis(); // keep track of time of start
   dstate = 1; // go to waiting state
   break;  
   
   case 1: // waiting for results to be ready
   if (millis() - Dtime > D_MILLIS) {dstate = 2;} // done waiting if true
   break;
   
   case 2: // read results into our array, and restart measurments
   for (i=0;i<Ddevices;i++)
   {
     DtempsF[i] = sensors.getTempFByIndex(i);
   }
   Dflag = 1;
   dstate = 0; // get another
   break;
  }
  
}

void setup(void)
{
  // start serial port
  Serial.begin(115200);
  Serial.println("Dallas Temperature IC Control Library Demo");
  setup_DS18B20();
}


void loop(void)
{ 
  Dstate(); // do measurements
  if (Dflag)
  {
   for (i=0;i<Ddevices;i++)
   {
   
    Serial.print("Temperature F #");
    Serial.print(i);
    Serial.print (": ");
    Serial.println(DtempsF[i]); // Why "byIndex"? You can have more than one IC on the same bus. 0 refers to the first IC on the wire
   }
   Dflag = 0;  
  }
}
